@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: FWObject.cpp,v 1.10 2006/05/17 04:16:34 vkurland Exp $
+  $Id: FWObject.cpp,v 1.13 2006/09/15 05:30:33 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -170,7 +170,7 @@ FWObject::FWObject(const FWObject &c) : list<FWObject*>(c)
     *this=c;
 }
 
-FWObject::FWObject(const FWObject *root)
+FWObject::FWObject(const FWObject *root,bool prepopulate)
 {
     init        = false;
     ref_counter = 0    ;
@@ -290,9 +290,18 @@ FWObject& FWObject::duplicate(const FWObject *x, bool preserve_id) throw(FWExcep
 {
     checkReadOnly();
     bool xro=x->getBool("ro"); 
+#if 0
+    cerr << "--------------------------------" << endl;
+    cerr << "this: " << endl;
+    dump(true,true);
+    cerr << endl;
+    cerr << "x: " << endl;
+    ((FWObject*)x)->dump(true,true);
+    cerr << endl;
+#endif
     shallowDuplicate(x,preserve_id);
 
-    destroyChildren();
+    destroyChildren();   // does it erase index entries?
 
     for(list<FWObject*>::const_iterator m=x->begin(); m!=x->end(); ++m) 
     {
@@ -311,7 +320,9 @@ FWObject* FWObject::addCopyOf(const FWObject *x, bool preserve_id) throw(FWExcep
     FWObject *o1;
     FWObjectDatabase *root = getRoot();
     if (root==NULL) root = x->getRoot();
-    o1 = root->create(x->getTypeName());
+    // do not prepopulte children for objects that do that automatically
+    // in their constructor
+    o1 = root->create(x->getTypeName(),"",false);
     if(!o1)
         throw FWException(string("Error creating object with type: ")+
                           x->getTypeName());
@@ -333,7 +344,7 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id) throw(
 
     data     = x->data;
     if (getBool("ro")) setReadOnly(false);
-    if(!preserve_id)
+    if (!preserve_id)
     {
         ref_counter = 0           ;
         xml_name    = x->xml_name ;
@@ -342,6 +353,8 @@ FWObject& FWObject::shallowDuplicate(const FWObject *x, bool preserve_id) throw(
 
     if (dbroot==NULL) setRoot(x->getRoot());
 
+    getRoot()->addToIndex(this);
+
     setDirty(true);
     return *this;
 }
@@ -366,7 +379,6 @@ FWObject*  FWObject::getLibrary() const
 {
     const FWObject *p=this;
     while (p!=NULL && !Library::isA(p) ) p=p->getParent();
-    assert (Library::isA(p));
     return (FWObject*)p;
 }
 
@@ -522,7 +534,7 @@ void FWObject::dump(bool recursive,bool brief,int offset)
     dump(cerr,recursive,brief,offset);
 }
 
-void   FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
+void FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
 {
     FWObject *o;
     string    n;
@@ -533,10 +545,14 @@ void   FWObject::dump(std::ostream &f,bool recursive,bool brief,int offset)
 	f << " ID="  << getId();
 	f << " Name=" << getName();
 	f << " Type=" << getTypeName();
-	f << " Library=" << getLibrary();
+        if (this!=getRoot())
+            f << " Library=" << getLibrary();
+	f << " Root=" << getRoot();
+        f << " ref_counter=" << ref_counter;
 
 	if (FWReference::cast(this)!=0) {
 	    f << " Ref=" << FWReference::cast(this)->getPointer();
+	    f << " RefID=" << FWReference::cast(this)->getPointerId();
 	}
 
 	f << endl;
@@ -618,7 +634,7 @@ void FWObject::add(FWObject *obj,bool validate)
 FWReference* FWObject::createRef()
 {
 //    FWObjectReference *ref=new FWObjectReference();
-    FWObjectReference *ref=FWObjectReference::cast(getRoot()->create(FWObjectReference::TYPENAME, true));
+    FWObjectReference *ref=FWObjectReference::cast(getRoot()->create(FWObjectReference::TYPENAME));
     ref->setPointer(this);
     return ref;
 }
@@ -839,10 +855,12 @@ bool FWObject::validateChild(FWObject *obj)
 
 void FWObject::destroyChildren()
 {
+    FWObjectDatabase *dbroot = getRoot();
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
         FWObject *o=*m;
         o->destroyChildren();
+        if (dbroot) dbroot->removeFromIndex( o->getId() );
         delete o;
     }
     clear();
@@ -871,6 +889,8 @@ void FWObject::destroyChildren()
  */
 void FWObject::clearChildren(bool recursive)
 {
+    FWObjectDatabase *dbroot = getRoot();
+
     checkReadOnly();
 
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
@@ -879,7 +899,10 @@ void FWObject::clearChildren(bool recursive)
         if (recursive) o->clearChildren(recursive);
         o->unref();
         if(o->ref_counter==0) 
+        {
+            if (dbroot) dbroot->removeFromIndex( o->getId() );
             delete o;
+        }
     }
     clear();
     setDirty(true);
@@ -891,10 +914,13 @@ void FWObject::clearChildren(bool recursive)
  */
 void FWObject::deleteChildren()
 {
+    FWObjectDatabase *dbroot = getRoot();
+
     for(list<FWObject*>::iterator m=begin(); m!=end(); ++m) 
     {
         FWObject *o=*m;
         o->deleteChildren();
+        if (dbroot) dbroot->removeFromIndex( o->getId() );
         delete o;
     }
     clear();
@@ -912,6 +938,15 @@ int FWObject::getChildrenCount()
 bool FWObject::isChildOf(FWObject *obj)
 {
     if (this==obj) return false;
+#if 0
+    cerr << "FWObject::isChildOf" << endl;
+    cerr << "this: " << endl;
+    dump(true,true);
+    cerr << endl;
+    cerr << "obj: " << endl;
+    obj->dump(true,true);
+    cerr << endl;
+#endif
     FWObject *p=this;
     while (p!=NULL && p!=obj) p=p->getParent();
     return (p==obj);
