@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: FWObjectDatabase.cpp,v 1.16 2006/04/23 15:45:25 vkurland Exp $
+  $Id: FWObjectDatabase.cpp,v 1.19 2006/09/15 05:30:33 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -119,6 +119,14 @@ FWObjectDatabase::FWObjectDatabase(FWObjectDatabase& d):FWObject()
     init=true;
     *this = d;  // copies entire tree
     setId( getRootId() );
+
+// I do not understand why do I need to reindex the whole database
+// after operator=. It calls FWobject::duplicate, which in turn
+// uses FWObject::shallowDuplicate, which calls addToIndex
+// This is necessary however.
+//
+    addToIndexRecursive( this );  // reindex
+
     setDirty(false);
     init=false;
 }
@@ -291,18 +299,6 @@ xmlNodePtr FWObjectDatabase::toXML(xmlNodePtr parent) throw(FWException)
     return parent;
 }
 
-FWObject *FWObjectDatabase::createFromXML(xmlNodePtr data)
-{
-    FWObject *nobj;
-    const char *n=FROMXMLCAST(data->name);
-    if (!n) 
-        return NULL;
-    
-    nobj=create(n);
-    
-    return(nobj);
-}
-
 string FWObjectDatabase::generateUniqueId()
 {
     char res[20];
@@ -341,9 +337,20 @@ void FWObjectDatabase::addToIndex(FWObject* o)
     }
 }
 
-FWObject* FWObjectDatabase::findInIndex(const std::string &id)
+void FWObjectDatabase::removeFromIndex(const string &id)
+{
+    obj_index.erase(id);
+}
+
+FWObject* FWObjectDatabase::checkIndex(const std::string &id)
 {
     FWObject *o = obj_index[id];
+    return o;
+}
+
+FWObject* FWObjectDatabase::findInIndex(const std::string &id)
+{
+    FWObject *o = checkIndex(id);
     if (o!=NULL) index_hits++;
     else
     {
@@ -359,7 +366,38 @@ FWObject* FWObjectDatabase::findInIndex(const std::string &id)
 void FWObjectDatabase::buildIndex()
 {
     clearIndex();
-    _addToIndexRecursive(this);
+    addToIndexRecursive(this);
+}
+
+void FWObjectDatabase::_clearReferenceCounters(FWObject *o)
+{
+    o->clearRefCounter();
+    for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
+    {
+        _clearReferenceCounters(*i);
+    }
+}
+
+void FWObjectDatabase::_fixReferenceCounters(FWObject *o)
+{
+    if (FWReference::cast(o)!=NULL)
+    {
+        FWObject *obj = findInIndex( FWReference::cast(o)->getPointerId() );
+        assert(obj);
+        obj->ref();
+    }
+    for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
+    {
+        (*i)->ref();
+        _fixReferenceCounters(*i);
+    }
+}
+
+void FWObjectDatabase::reIndex()
+{
+    buildIndex();
+    _clearReferenceCounters(this);
+    _fixReferenceCounters(this);
 }
 
 void FWObjectDatabase::clearIndex()
@@ -375,23 +413,52 @@ void FWObjectDatabase::getIndexStats(int &index_size, int &hit_counter, int &mis
     miss_counter = index_misses;
 }
 
-void FWObjectDatabase::_addToIndexRecursive(FWObject *o)
+void FWObjectDatabase::addToIndexRecursive(FWObject *o)
 {
     addToIndex(o);
     for (FWObject::iterator i=o->begin(); i!=o->end(); ++i)
-        _addToIndexRecursive(*i);
+        addToIndexRecursive(*i);
 }
 
-#define CREATE_OBJ(classname,str) \
+/*
+ ***********************************************************************
+ */
+FWObject *FWObjectDatabase::createFromXML(xmlNodePtr data)
+{
+    const char *n;
+    string typen, id;
+
+    n=FROMXMLCAST(data->name);
+    if(!n)
+        return NULL;
+    typen = n;
+
+    n=FROMXMLCAST(xmlGetProp(data,TOXMLCAST("id")));
+    if(n)
+    {
+        id = n;
+        FREEXMLBUFF(id);
+    }
+
+// create new object but do not prepopulate objects that
+// automatically create children in constructor
+    return create(typen,id,false);   
+}
+
+
+#define CREATE_OBJ(classname,str,id,prepopulate)     \
 if(strcmp(classname::TYPENAME,str)==SAME) \
 { \
-  if (create_with_root) nobj=new classname(this); else nobj=new classname(); \
+  nobj=new classname(this,prepopulate); \
+  if (!id.empty()) nobj->setId(id); \
   addToIndex(nobj); \
   return nobj; \
 }
 
 
-FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_root)
+FWObject *FWObjectDatabase::create(const string &type_name,
+                                   const std::string &id,
+                                   bool prepopulate)
 {
     const char *n=type_name.c_str();
     FWObject   *nobj;
@@ -400,7 +467,8 @@ FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_roo
 
     if(strcmp("AnyNetwork"  ,n)==SAME)
     {
-        nobj = new Network();
+        nobj = new Network(this,prepopulate);
+        if (!id.empty()) nobj->setId(id);
         nobj->setXMLName("AnyNetwork");
         addToIndex(nobj);
         return nobj;
@@ -408,7 +476,8 @@ FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_roo
 
     if(strcmp("AnyIPService",n)==SAME)
     {
-        nobj = new IPService();
+        nobj = new IPService(this,prepopulate);
+        if (!id.empty()) nobj->setId(id);
         nobj->setXMLName("AnyIPService");
         addToIndex(nobj);
         return nobj;
@@ -416,66 +485,67 @@ FWObject *FWObjectDatabase::create(const string &type_name, bool create_with_roo
 
     if(strcmp("AnyInterval" ,n)==SAME)
     {
-        nobj = new Interval();
+        nobj = new Interval(this,prepopulate);
+        if (!id.empty()) nobj->setId(id);
         nobj->setXMLName("AnyInterval");
         addToIndex(nobj);
         return nobj;
     }
 
 
-    CREATE_OBJ(Library             ,n);
-    CREATE_OBJ(Policy              ,n);
-    CREATE_OBJ(InterfacePolicy     ,n);
-    CREATE_OBJ(NAT                 ,n);
-    CREATE_OBJ(Routing             ,n);
-    CREATE_OBJ(PolicyRule          ,n);
-    CREATE_OBJ(NATRule             ,n);
-    CREATE_OBJ(RoutingRule         ,n);
-    CREATE_OBJ(RuleElementSrc      ,n);
-    CREATE_OBJ(RuleElementDst      ,n);
-    CREATE_OBJ(RuleElementSrv      ,n);
-    CREATE_OBJ(RuleElementItf      ,n);
-    CREATE_OBJ(RuleElementOSrc     ,n);
-    CREATE_OBJ(RuleElementODst     ,n);
-    CREATE_OBJ(RuleElementOSrv     ,n);
-    CREATE_OBJ(RuleElementTSrc     ,n);
-    CREATE_OBJ(RuleElementTDst     ,n);
-    CREATE_OBJ(RuleElementTSrv     ,n);
-    CREATE_OBJ(RuleElementInterval ,n);
-    CREATE_OBJ(RuleElementRDst     ,n);
-    CREATE_OBJ(RuleElementRGtw     ,n);
-    CREATE_OBJ(RuleElementRItf     ,n);
-    CREATE_OBJ(HostOptions         ,n);
-    CREATE_OBJ(FirewallOptions     ,n);
-    CREATE_OBJ(PolicyRuleOptions   ,n);
-    CREATE_OBJ(NATRuleOptions      ,n);
-    CREATE_OBJ(RoutingRuleOptions  ,n);
-    CREATE_OBJ(Host                ,n);
-    CREATE_OBJ(AddressRange        ,n);
-    CREATE_OBJ(Management          ,n);
-    CREATE_OBJ(PolicyInstallScript ,n);
-    CREATE_OBJ(SNMPManagement      ,n);
-    CREATE_OBJ(FWBDManagement      ,n);
-    CREATE_OBJ(Firewall            ,n);
-    CREATE_OBJ(Network             ,n);
-    CREATE_OBJ(Interface           ,n);
-    CREATE_OBJ(IPv4                ,n);
-    CREATE_OBJ(physAddress         ,n);
-    CREATE_OBJ(DNSName             ,n);
-    CREATE_OBJ(AddressTable        ,n);
-    CREATE_OBJ(FWObjectReference   ,n);
-    CREATE_OBJ(FWServiceReference  ,n);
-    CREATE_OBJ(FWIntervalReference ,n);
-    CREATE_OBJ(TCPService          ,n);
-    CREATE_OBJ(UDPService          ,n);
-    CREATE_OBJ(TagService          ,n);
-    CREATE_OBJ(ICMPService         ,n);
-    CREATE_OBJ(IPService           ,n);
-    CREATE_OBJ(CustomService       ,n);
-    CREATE_OBJ(Interval            ,n);
-    CREATE_OBJ(ObjectGroup         ,n);
-    CREATE_OBJ(ServiceGroup        ,n);
-    CREATE_OBJ(IntervalGroup       ,n);
+    CREATE_OBJ(Library             ,n, id, prepopulate);
+    CREATE_OBJ(Policy              ,n, id, prepopulate);
+    CREATE_OBJ(InterfacePolicy     ,n, id, prepopulate);
+    CREATE_OBJ(NAT                 ,n, id, prepopulate);
+    CREATE_OBJ(Routing             ,n, id, prepopulate);
+    CREATE_OBJ(PolicyRule          ,n, id, prepopulate);
+    CREATE_OBJ(NATRule             ,n, id, prepopulate);
+    CREATE_OBJ(RoutingRule         ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementSrc      ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementDst      ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementSrv      ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementItf      ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementOSrc     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementODst     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementOSrv     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementTSrc     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementTDst     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementTSrv     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementInterval ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementRDst     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementRGtw     ,n, id, prepopulate);
+    CREATE_OBJ(RuleElementRItf     ,n, id, prepopulate);
+    CREATE_OBJ(HostOptions         ,n, id, prepopulate);
+    CREATE_OBJ(FirewallOptions     ,n, id, prepopulate);
+    CREATE_OBJ(PolicyRuleOptions   ,n, id, prepopulate);
+    CREATE_OBJ(NATRuleOptions      ,n, id, prepopulate);
+    CREATE_OBJ(RoutingRuleOptions  ,n, id, prepopulate);
+    CREATE_OBJ(Host                ,n, id, prepopulate);
+    CREATE_OBJ(AddressRange        ,n, id, prepopulate);
+    CREATE_OBJ(Management          ,n, id, prepopulate);
+    CREATE_OBJ(PolicyInstallScript ,n, id, prepopulate);
+    CREATE_OBJ(SNMPManagement      ,n, id, prepopulate);
+    CREATE_OBJ(FWBDManagement      ,n, id, prepopulate);
+    CREATE_OBJ(Firewall            ,n, id, prepopulate);
+    CREATE_OBJ(Network             ,n, id, prepopulate);
+    CREATE_OBJ(Interface           ,n, id, prepopulate);
+    CREATE_OBJ(IPv4                ,n, id, prepopulate);
+    CREATE_OBJ(physAddress         ,n, id, prepopulate);
+    CREATE_OBJ(DNSName             ,n, id, prepopulate);
+    CREATE_OBJ(AddressTable        ,n, id, prepopulate);
+    CREATE_OBJ(FWObjectReference   ,n, id, prepopulate);
+    CREATE_OBJ(FWServiceReference  ,n, id, prepopulate);
+    CREATE_OBJ(FWIntervalReference ,n, id, prepopulate);
+    CREATE_OBJ(TCPService          ,n, id, prepopulate);
+    CREATE_OBJ(UDPService          ,n, id, prepopulate);
+    CREATE_OBJ(TagService          ,n, id, prepopulate);
+    CREATE_OBJ(ICMPService         ,n, id, prepopulate);
+    CREATE_OBJ(IPService           ,n, id, prepopulate);
+    CREATE_OBJ(CustomService       ,n, id, prepopulate);
+    CREATE_OBJ(Interval            ,n, id, prepopulate);
+    CREATE_OBJ(ObjectGroup         ,n, id, prepopulate);
+    CREATE_OBJ(ServiceGroup        ,n, id, prepopulate);
+    CREATE_OBJ(IntervalGroup       ,n, id, prepopulate);
 
     cerr << "FWObjectDatabase::create: ERROR: attempt to create object of unknown type '" 
          << type_name << "'" << endl;
@@ -606,7 +676,7 @@ void FWObjectTreeScanner::addRecursively(FWObject *src)
 
     if (dstMap[src->getId()]==NULL)
     {
-        FWObject *o1=treeRoot->create(src->getTypeName());
+        FWObject *o1=treeRoot->create(src->getTypeName(),"",false);
         FWObject *pdst = dstMap[src->getParent()->getId()];
         assert(pdst!=NULL);
 
@@ -692,6 +762,8 @@ void FWObjectTreeScanner::scanAndAdd(FWObject *dst,FWObject *source)
     }
 }
 
+//#define DEBUG_MERGE 1
+
 void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 {
     string dobjId             = FWObjectDatabase::getDeletedObjectsId();
@@ -766,6 +838,7 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 //        if ((*i)->getId()==dobjId) continue;
 
         FWObject *dobj;
+        FWObject *sobj;
         if (FWReference::cast( *i ))
         {
             FWReference *r=FWReference::cast(*i);
@@ -774,14 +847,29 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
 
         if (dobj==NULL)
         {
-            FWObject *o1=treeRoot->create( (*i)->getTypeName());
+            sobj = *i;
+            FWObject *o1=treeRoot->create( sobj->getTypeName());
 
             FWObject *pdst = dstMap[ src->getId() ];
             assert(pdst!=NULL);
 
             pdst->add(o1);
 
-            o1->duplicate( (*i), false); // copy IDs as well
+#ifdef DEBUG_MERGE
+            cerr << "--------------------------------" << endl;
+            cerr << "merge: duplicate #1 " << endl;
+            cerr << "dobj: " << endl;
+            o1->dump(true,true);
+            cerr << endl;
+            cerr << "sobj: " << endl;
+            sobj->dump(true,true);
+#endif
+
+            o1->duplicate( sobj, false); // copy IDs as well
+
+#ifdef DEBUG_MERGE
+            cerr << "duplicate #1 done" << endl;
+#endif
 
 /* there may be children objects since we did a deep copy         */
             walkTree( dstMap , o1 );
@@ -822,13 +910,29 @@ void FWObjectTreeScanner::merge(FWObject *dst,FWObject *src)
                 if (firstChild==NULL || FWReference::cast(firstChild)!=NULL)
                 {
                     if (crp!=NULL && crp->askUser( dobj, *i ))
+                    {
+#ifdef DEBUG_MERGE
+                        cerr << "--------------------------------" << endl;
+                        cerr << "merge: duplicate #2 " << endl;
+                        dobj->dump(true,true);
+                        cerr << endl;
+#endif
                         dobj->duplicate( (*i), false );
+                    }
                 } else merge( dobj , *i );
             }
             else
             {
                 if (crp!=NULL && crp->askUser( dobj, *i ))
+                {
+#ifdef DEBUG_MERGE
+                    cerr << "--------------------------------" << endl;
+                    cerr << "merge: duplicate #3 " << endl;
+                    dobj->dump(true,true);
+                    cerr << endl;
+#endif
                     dobj->duplicate( (*i), false );
+                }
             }
         }
     }
@@ -844,7 +948,7 @@ FWObjectDatabase* FWObjectDatabase::exportSubtree( const list<FWObject*> &libs )
     for (list<FWObject*>::const_iterator i=libs.begin(); i!=libs.end(); i++)
     {
         FWObject *lib = *i;
-        FWObject *nlib = ndb->create(lib->getTypeName(),true);
+        FWObject *nlib = ndb->create(lib->getTypeName());
         ndb->add(nlib);
         *nlib = *lib;
     }
@@ -863,7 +967,7 @@ FWObjectDatabase* FWObjectDatabase::exportSubtree( FWObject *lib )
 
     ndb->init=true;
 
-    FWObject *nlib = ndb->create(lib->getTypeName(),true);
+    FWObject *nlib = ndb->create(lib->getTypeName());
     ndb->add(nlib);
     *nlib = *lib;
 
