@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: FWObjectDatabase.h,v 1.7 2006/04/23 15:45:26 vkurland Exp $
+  $Id: FWObjectDatabase.h,v 1.11 2006/09/15 05:30:33 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -62,7 +62,12 @@ class IDcounter {
  */
 class FWObjectDatabase : public FWObject
 {
-    protected:
+private:
+
+    void _clearReferenceCounters(FWObject *o);
+    void _fixReferenceCounters(FWObject *o);
+    
+protected:
 
     static const std::string         DTD_FILE_NAME ;
     time_t                           lastModified;
@@ -74,9 +79,8 @@ class FWObjectDatabase : public FWObject
     std::string data_file;
 
     Firewall* _findFirewallByNameRecursive(FWObject* db,const std::string &name) throw(FWException);
-    void _addToIndexRecursive(FWObject *o);
 
-    public:
+public:
 
     DECLARE_FWOBJECT_SUBTYPE(FWObjectDatabase);
 
@@ -94,11 +98,23 @@ class FWObjectDatabase : public FWObject
 
     // --- methods dealing with object index
 
+    void addToIndexRecursive(FWObject *o);
+
     /**
      * add an object to the index
      */
     void addToIndex(FWObject* obj);
 
+    /**
+     * remove an object from the index
+     */
+    void removeFromIndex(const std::string &id);
+
+    /**
+     * check if an object is present in the index using its ID
+     */
+    FWObject* checkIndex(const std::string &id);
+
     /**
      * find an object in the index using its ID
      */
@@ -114,6 +130,13 @@ class FWObjectDatabase : public FWObject
      */
     void clearIndex();
 
+    /**
+     * this is just like buildIndex, except it also fixes reference
+     * counters in objects. Call this method after loading database
+     * from XML file. This method uses private _fixReferenceCounters
+     */
+    void reIndex();
+    
     /**
      * return index usage statistics
      */
@@ -129,7 +152,7 @@ class FWObjectDatabase : public FWObject
      * objects using constructor that uses pointer to this as a parameter,
      * otherwise empty constructor is used
      */
-    virtual FWObject *create(const std::string &type, bool create_with_root=false);
+    FWObject *create(const std::string &type, const std::string &id="", bool prepopulate=true);
 
     /**
      * Creates instance of FWObject using its XML representation
@@ -167,7 +190,7 @@ class FWObjectDatabase : public FWObject
     class ConflictResolutionPredicate
     {
         public:
-
+        virtual ~ConflictResolutionPredicate() {}
         virtual bool askUser(FWObject *o1,FWObject *o2) 
         { 
             return true;
