@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Rule.cpp,v 1.12 2006/06/03 18:31:36 vkurland Exp $
+  $Id: Rule.cpp,v 1.14 2006/09/10 22:40:37 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -55,7 +55,7 @@ Rule::Rule()
     interface_id="";
 }
 
-Rule::Rule(const FWObject *root) : Group(root)
+Rule::Rule(const FWObject *root,bool prepopulate) : Group(root,prepopulate)
 {
     remStr("name");
     setInt("position",0);
@@ -97,21 +97,24 @@ PolicyRule::PolicyRule()
     setStr("action","Deny");
 }
 
-PolicyRule::PolicyRule(const FWObject *root) : Rule(root)
+PolicyRule::PolicyRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
     setStr("action","Deny");
 
-    FWObject         *re;
-    FWObjectDatabase *db=(FWObjectDatabase*)root;
-    assert(db);
+    if (prepopulate)
+    {
+        FWObject         *re;
+        FWObjectDatabase *db=(FWObjectDatabase*)root;
+        assert(db);
 
-    re=db->create("Src",true);  assert(re!=NULL); add(re);
-    re=db->create("Dst",true);  assert(re!=NULL); add(re);
-    re=db->create("Srv",true);  assert(re!=NULL); add(re);
-    re=db->create("When",true); assert(re!=NULL); add(re);
-    re=db->create("Itf",true);  assert(re!=NULL); add(re);
+        re=db->create("Src");  assert(re!=NULL); add(re);
+        re=db->create("Dst");  assert(re!=NULL); add(re);
+        re=db->create("Srv");  assert(re!=NULL); add(re);
+        re=db->create("When"); assert(re!=NULL); add(re);
+        re=db->create("Itf");  assert(re!=NULL); add(re);
 
-    add( db->create(PolicyRuleOptions::TYPENAME,true) );
+        add( db->create(PolicyRuleOptions::TYPENAME) );
+    }
 }
 
 FWObject& PolicyRule::shallowDuplicate(const FWObject *x, bool preserve_id) throw(FWException)
@@ -343,26 +346,28 @@ NATRule::NATRule() : Rule()
     rule_type=Unknown;
 }
 
-NATRule::NATRule(const FWObject *root) : Rule(root)
+NATRule::NATRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
     rule_type=Unknown;
 
-    FWObject         *re;
-    FWObjectDatabase *db=(FWObjectDatabase*)(root);
-    assert(db);
+    if (prepopulate)
+    {
+        FWObject         *re;
+        FWObjectDatabase *db=(FWObjectDatabase*)(root);
+        assert(db);
 
-    re=db->create("OSrc",true);  assert(re!=NULL); add(re);
-    re=db->create("ODst",true);  assert(re!=NULL); add(re);
-    re=db->create("OSrv",true);  assert(re!=NULL); add(re);
+        re=db->create("OSrc");  assert(re!=NULL); add(re);
+        re=db->create("ODst");  assert(re!=NULL); add(re);
+        re=db->create("OSrv");  assert(re!=NULL); add(re);
 
-    re=db->create("TSrc",true);  assert(re!=NULL); add(re);
-    re=db->create("TDst",true);  assert(re!=NULL); add(re);
-    re=db->create("TSrv",true);  assert(re!=NULL); add(re);
+        re=db->create("TSrc");  assert(re!=NULL); add(re);
+        re=db->create("TDst");  assert(re!=NULL); add(re);
+        re=db->create("TSrv");  assert(re!=NULL); add(re);
 
-    add( db->create(NATRuleOptions::TYPENAME,true) );
+        add( db->create(NATRuleOptions::TYPENAME) );
+    }
 }
 
-
 RuleElementOSrc*  NATRule::getOSrc()  const
 {
     return RuleElementOSrc::cast(getFirstByType(RuleElementOSrc::TYPENAME));
@@ -518,22 +523,24 @@ RoutingRule::RoutingRule() : Rule()
     setMetric(0);
 }
 
-RoutingRule::RoutingRule(const FWObject *root) : Rule(root)
+RoutingRule::RoutingRule(const FWObject *root,bool prepopulate) : Rule(root,prepopulate)
 {
-    FWObject         *re;
-    FWObjectDatabase *db=(FWObjectDatabase*)(root);
-    assert(db);
+    if (prepopulate)
+    {
+        FWObject         *re;
+        FWObjectDatabase *db=(FWObjectDatabase*)(root);
+        assert(db);
 
-    re=db->create("RDst",true);  assert(re!=NULL); add(re);
-    re=db->create("RGtw",true);  assert(re!=NULL); add(re);
-    re=db->create("RItf",true);  assert(re!=NULL); add(re);
+        re=db->create("RDst");  assert(re!=NULL); add(re);
+        re=db->create("RGtw");  assert(re!=NULL); add(re);
+        re=db->create("RItf");  assert(re!=NULL); add(re);
     
-    setMetric(0);
+        setMetric(0);
 
-    add( db->create(RoutingRuleOptions::TYPENAME,true) );
+        add( db->create(RoutingRuleOptions::TYPENAME) );
+    }
 }
 
-
 RuleElementRDst*  RoutingRule::getRDst()  const
 {
     return RuleElementRDst::cast(getFirstByType(RuleElementRDst::TYPENAME));
