@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: dns.cpp,v 1.4 2005/03/21 06:34:19 vkurland Exp $
+  $Id: dns.cpp,v 1.8 2006/06/25 04:31:26 vkurland Exp $
 
 
   This program is free software which we release under the GNU General Public
@@ -60,6 +60,8 @@
 using namespace std;
 using namespace libfwbuilder;
 
+#undef DNS_DEBUG
+
 const size_t DNS::RSP_BUF_SIZE=64*1024;
 
 Mutex *DNS::gethostbyname_mutex;
@@ -291,7 +293,7 @@ HostEnt DNS::getHostByAddr(const IPAddress &addr) throw(FWException)
     return v;
 }
 
-vector<IPAddress> DNS::getHostByName(const string &name) throw(FWException)
+list<IPAddress> DNS::getHostByName(const string &name) throw(FWException)
 {
     struct hostent hostbuf;
     struct hostent *hp=NULL;
@@ -355,7 +357,7 @@ vector<IPAddress> DNS::getHostByName(const string &name) throw(FWException)
 
 #endif
     
-    vector<IPAddress> v;
+    list<IPAddress> v;
     try
     {
         for(char **p = hp->h_addr_list; *p != 0; p++) 
@@ -383,11 +385,15 @@ vector<IPAddress> DNS::getHostByName(const string &name) throw(FWException)
     gethostbyname_mutex->unlock();
 #endif
 
+    v.sort();
+
     return v;
 }
 
 multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,SyncFlag *stop_program, int retries_, int timeout_) throw(FWException)
 {
+    std::ostringstream str;
+
 #if !defined(HAVE_GOODLIBRESOLV)
     throw FWException("This feature is not implemented on your OS.");
 #else
@@ -399,11 +405,13 @@ multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,Sync
     res.retrans = timeout_;
     res.retry   = retries_;
 
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
 
     auto_ptr<unsigned char> answer(new unsigned char[RSP_BUF_SIZE]);
     
-    *logger << "Requesting list of name servers for domain '" << domain << "'"  << '\n';
+    str << "Requesting list of name servers for domain '" << domain << "'"  << '\n';
+    *logger << str;
+
     int  len = res_nquery(&res, 
                           domain.c_str(), 
                           ns_c_in,
@@ -411,7 +419,7 @@ multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,Sync
                           answer.get(), 
                           RSP_BUF_SIZE);
 
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     
     if(len<0)
         throw FWException("Error returned while quering domain NS records");
@@ -424,19 +432,19 @@ multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,Sync
     if(ns_initparse(answer.get(), len, &handle) < 0) 
         throw FWException("Zone parse error in initparse");
     
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
 
     if(ns_msg_getflag(handle, ns_f_rcode) != ns_r_noerror)
         throw FWException(getErrorMessage(ns_msg_getflag(handle, ns_f_rcode)));
     
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
 
     if(ns_msg_count(handle, ns_s_an) == 0)
         throw FWException("Answer contains to records");
     
     while(true) 
     {        
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
         ns_rr rr;
         if(ns_parserr(&handle, ns_s_an, -1, &rr))
         {
@@ -446,21 +454,23 @@ multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,Sync
                 break;
         }
         
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
         if(ns_rr_type(rr)==ns_t_ns && ns_rr_class(rr)==ns_c_in)
         {
             char dn[NS_MAXDNAME];
             if(dn_expand(answer.get(), answer.get() + len, ns_rr_rdata(rr), dn, sizeof(dn))<0)
                 throw FWException("A record parse error in parserr");
-            CHECK_STOP
-            vector<IPAddress> a=DNS::getHostByName(dn);
-            CHECK_STOP
-            for(vector<IPAddress>::iterator i=a.begin();i!=a.end();++i)
+            CHECK_STOP_AND_THROW_EXCEPTION
+            list<IPAddress> a=DNS::getHostByName(dn);
+            CHECK_STOP_AND_THROW_EXCEPTION
+            for(list<IPAddress>::iterator i=a.begin();i!=a.end();++i)
                 v.insert(pair<string, IPAddress>(string(dn), (*i)));
         } 
     }
     
-    *logger << "Succesfuly found " << (int)v.size() << " name servers."  << '\n';
+    str << "Succesfuly found " << (int)v.size() << " name servers."  << '\n';
+    *logger << str;
+
     return v;
 #endif
 }
@@ -472,6 +482,7 @@ multimap<string, IPAddress> DNS::getNS(const string &domain, Logger *logger,Sync
  */
 map<string, set<IPAddress> > DNS::findA(const string &domain, Logger *logger,SyncFlag *stop_program, int retries_, int timeout_) throw(FWException)
 {
+    std::ostringstream str;
 #if !defined(HAVE_GOODLIBRESOLV)
     throw FWException("This feature is not implemented on your OS.");
 #else
@@ -483,7 +494,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, Logger *logger,Syn
     if(!ns.size())
         throw FWException("No NS records found");
 
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     timeout.check();
 
     FWException *last_err=NULL; // to avoid compiler warning
@@ -495,10 +506,11 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, Logger *logger,Syn
             timeout.check();
         } catch(FWException &ex)
         {
-            *logger << "Quering NS " << (*nsi).second.toString() 
-                    << " ( " << (*nsi).first << " ) " 
-                    << " failed. with error: '" << ex.toString() << "'" << '\n' 
-                    << " Cycling to next one" << '\n';
+            str << "Quering NS " << (*nsi).second.toString() 
+                << " ( " << (*nsi).first << " ) " 
+                << " failed. with error: '" << ex.toString() << "'" << '\n' 
+                << " Cycling to next one" << '\n';
+            *logger << str;
             last_err = new FWException(ex) ;
         }
     }
@@ -519,13 +531,15 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, Logger *logger,Syn
  */
 map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &ns, Logger *logger,SyncFlag *stop_program, int retries_, int timeout_) throw(FWException)
 {
+    std::ostringstream str;
 #if !defined(HAVE_GOODLIBRESOLV)
     throw FWException("This feature is not implemented on your OS.");
 #else
     TimeoutCounter timeout(timeout_, "Getting A records");
 
-    *logger << "Querying server: " << ns.toString() << '\n';
-    
+    str << "Querying server: " << ns.toString() << '\n';
+    *logger << str;
+
     struct __res_state res;
     
     if(res_ninit(&res)==-1)
@@ -543,7 +557,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
     res.nsaddr_list[0].sin_port   = htons(NAMESERVER_PORT);
     res.nsaddr_list[0].sin_addr   = nsaddress;
 
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     timeout.check();
 
     // Create a query packet for the requested zone name.
@@ -553,7 +567,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
                               ns_c_in, ns_t_axfr, NULL,
                               0, 0, buf, sizeof buf);
     
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     timeout.check();
 
     if(msglen<0)
@@ -568,7 +582,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
     if (connect(sockFD, (struct sockaddr *)sin, sizeof *sin) < 0) 
         throw FWException("Error connecting to DNS server");
     
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     timeout.check();
 
     // Send length & message for zone transfer
@@ -580,12 +594,12 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
             
     try
     {
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
 	timeout.check();
 
         if(write(sockFD, tmp, NS_INT16SZ) != NS_INT16SZ)
             throw FWException("Error 01 sending AXFR query to DNS server");
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
 	timeout.check();
 
         *logger << "Sending Query" << '\n';
@@ -599,7 +613,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
         *logger << "Reading response." << '\n';
         while(!done)
         {
-            CHECK_STOP
+            CHECK_STOP_AND_THROW_EXCEPTION
 	    timeout.check();
 
             // Read the length of the response.
@@ -607,7 +621,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
             size_t amtToRead = INT16SZ;
             while(amtToRead > 0)
             {
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
 		timeout.check();
 
 		int numRead=timeout.read(sockFD, cp, amtToRead);
@@ -624,7 +638,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
             else if(len<0)
                 throw FWException("Ivalid packet size returned");
 
-            CHECK_STOP
+            CHECK_STOP_AND_THROW_EXCEPTION
 	    timeout.check();
 
             try
@@ -640,7 +654,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
             cp = answer.get();
             while(amtToRead > 0)
             {
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
 		timeout.check();
 
 		int numRead=timeout.read(sockFD, cp, amtToRead);
@@ -656,7 +670,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
 
             ns_msg handle;
                     
-            CHECK_STOP
+            CHECK_STOP_AND_THROW_EXCEPTION
 	    timeout.check();
 
             if(ns_initparse(answer.get(), len, &handle) < 0) 
@@ -670,7 +684,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
                     
             while(true) 
             {
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
 		timeout.check();
 
                 ns_rr rr;
@@ -682,7 +696,7 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
                         break;
                 }
                         
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
 		timeout.check();
 
                 if(ns_rr_type(rr)==ns_t_a && ns_rr_class(rr)==ns_c_in)
@@ -719,7 +733,8 @@ map<string, set<IPAddress> > DNS::findA(const string &domain, const IPAddress &n
         (void) close(sockFD);
         throw;
     }
-    *logger << "Succesfuly found " << (int)v.size() << " hosts."  << '\n';
+    str << "Succesfuly found " << (int)v.size() << " hosts."  << '\n';
+    *logger << str;
     return v;
 #endif
 }
@@ -733,7 +748,12 @@ DNS_getNS_query::DNS_getNS_query(const string &domain_, int retries_, int timeou
 
 void DNS_getNS_query::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWException)
 {
-    result = DNS::getNS(domain, logger,stop_program, retries, timeout);
+    try
+    {
+        result = DNS::getNS(domain, logger,stop_program, retries, timeout);
+    } catch (FWException &ex)
+    {
+    }
 }
 
 DNS_findA_query::DNS_findA_query()
@@ -758,7 +778,12 @@ void DNS_findA_query::init(const string &domain_, const IPAddress &ns_, int retr
 
 void DNS_findA_query::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWException)
 {
-    result = findA(domain, ns, logger,stop_program, retries, timeout);
+    try
+    {
+        result = findA(domain, ns, logger,stop_program, retries, timeout);
+    } catch (FWException &ex)
+    {
+    }
 }
 
 DNS_bulkBackResolve_query::DNS_bulkBackResolve_query(set<IPAddress> _ips, 
@@ -783,6 +808,7 @@ DNS_bulkBackResolve_query::~DNS_bulkBackResolve_query()
 
 void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
 {
+    std::ostringstream str;
     void **void_pair=(void**)args;
     DNS_bulkBackResolve_query *p      = static_cast<DNS_bulkBackResolve_query*>(void_pair[0]);
     Logger                    *logger = static_cast<Logger *>(void_pair[1]);
@@ -791,7 +817,13 @@ void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
     {
         while( true )
         {
-            CHECK_STOP
+            stop_program->lock();
+            if ( stop_program->peek() )
+            {
+                stop_program->unlock();
+                return NULL;
+            }
+            stop_program->unlock();
 
             p->queue_mutex.lock();
             if(p->ips.empty())
@@ -799,14 +831,13 @@ void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
                 p->queue_mutex.unlock();
                 break;
             }
-            IPAddress j=p->ips.front(); p->ips.pop();
-
-	    *logger << "Trying " << j.toString() 
-		    << ";   " << int(p->ips.size()) << " more in the queue\n";
-
+            IPAddress j=p->ips.front();
+            p->ips.pop();
             p->queue_mutex.unlock();
         
-            CHECK_STOP
+	    str << "Trying " << j.toString() 
+                << ";   " << int(p->ips.size()) << " more in the queue\n";
+            *logger << str;
 
             try
             {
@@ -814,7 +845,7 @@ void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
                 HostEnt he=DNS::getHostByAddr(j, p->retries, p->timeout);
                 std::ostringstream str;
                 str << "Resolved  " << j.toString() << ": " << he.name << "\n";
-                *logger << str.str();
+                *logger << str;
                 p->result_mutex.lock();
                 p->result[j]=he;
                 p->result_mutex.unlock();
@@ -823,7 +854,7 @@ void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
                 std::ostringstream str;
                 str << "Could not resolve address " << j.toString() 
 		    << " to a host name, will use generic name\n";
-                *logger << str.str();
+                *logger << str;
                 p->failed_mutex.lock();
                 p->failed.insert(j);
                 p->failed_mutex.unlock();
@@ -835,13 +866,21 @@ void* libfwbuilder::DNS_bulkBackResolve_Thread(void *args)
         // do nothing, just break the "while" loop
     }
 
-    *logger << " Worker thread exiting\n";
+//#ifdef DNS_DEBUG
+    str << " Worker thread exiting. " << p->running_count << " threads left.\n";
+    *logger << str;
+//#endif
 
     p->running_mutex.lock   ();
     p->running_count--;
     p->running_cond.signal  ();
     p->running_mutex.unlock ();
 
+//#ifdef DNS_DEBUG
+    str << " Worker thread terminated.\n";
+    *logger << str;
+//#endif
+
     return NULL;
 }
 
@@ -857,15 +896,20 @@ void DNS_bulkBackResolve_query::run_impl(Logger *logger,SyncFlag *stop_program)
         void_pair[1]     = logger;
         void_pair[2]     = stop_program;
 
-	*logger << "  Creating worker thread " << "\n";
-
+#ifdef DNS_DEBUG
+	*logger << "  Creating worker thread\n";
+#endif
         pthread_t tid;
         pthread_attr_setdetachstate(&tattr, PTHREAD_CREATE_DETACHED);
         running_count++;
         pthread_create(&tid, NULL, DNS_bulkBackResolve_Thread, void_pair);
     }
 
-    running_mutex.lock();
+#ifdef DNS_DEBUG
+    std::ostringstream str;
+    str << "  Created " << running_count << " worker threads" << "\n";
+    *logger << str;
+#endif
 
     queue_mutex.unlock();
 
@@ -883,15 +927,37 @@ void DNS_bulkBackResolve_query::run_impl(Logger *logger,SyncFlag *stop_program)
  *  locked again when thread is woken up. This ensures that we always
  *  check running_count when mutex is locked. Threads use the same
  *  mutex when modify running_count.
+ *
+ *  Note 01/20/06:
+ *
+ *   The comment above is correct, but the code did not follow described
+ *   model.
+ *   method Cond::wait locks the mutex and calls pthread_cond_wait
+ *   This is because pthread_cond_wait atomically unlocks the mutex
+ *   and puts the thread into sleep mode waiting for for the condition
+ *   variable to be signaled. Also once signal is caught, before
+ *   returning to the calling thread, pthread_cond_wait locks the  mutex
+ *   again. Since we do nothing that would require this mutex here (we only
+ *   use this mechanism to make sure we get control back when all
+ *   worker threads terminate), we just unlock it and proceed with the loop.
  */
-    while(running_count)
+    while(1)
     {
+        CHECK_STOP_AND_RETURN
+
+        running_mutex.lock();
+        if (running_count<=0)
+        {
+            running_mutex.unlock();
+            break;
+        }
+        running_mutex.unlock();
+
         running_cond.wait(running_mutex);
+        running_mutex.unlock();
     }
 
-    running_mutex.unlock();
-
-    CHECK_STOP
+    CHECK_STOP_AND_RETURN
 
     if(!failed.empty())
         throw FWException("Some of IPs did not resolve");
