@@ -6,7 +6,7 @@
 
   Author:  Vadim Zaliva lord@crocodile.org
 
-  $Id: snmp.cpp,v 1.6 2005/05/03 05:23:37 vkurland Exp $
+  $Id: snmp.cpp,v 1.9 2006/06/25 04:31:26 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -151,6 +151,13 @@ const int SNMPCrawler::PTP_INTERFACE_TYPES[]={
    
 };
 
+/*
+ * temporary database object, needs this to be able to create
+ * interfaces and then add ip addresses to them
+ */
+static FWObjectDatabase snmp_tmp_db;
+
+
 #include <iostream>
 
 SNMPQuery::SNMPQuery(string h,string c, int retries_, long timeout_)
@@ -193,26 +200,26 @@ void SNMPQuery::fetchAll(Logger *logger,SyncFlag *stop_program) throw(FWExceptio
         throw FWException("No SNMP hostname specified");
 
     SNMPConnection c(hostname, community);
-    *logger << "Connecting to " << hostname << "\n";
     c.connect(retries, timeout);
 
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     
     fetchSysInfo   (logger,stop_program, &c);
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     
     fetchInterfaces(logger,stop_program, &c);
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
     
     fetchArpTable  (logger,stop_program, &c);
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
 
     fetchRoutingTable  (logger,stop_program, &c);
-    CHECK_STOP
+    CHECK_STOP_AND_THROW_EXCEPTION
 }
 
 void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnection *connection) throw(FWException)
 {
+    std::ostringstream str;
     *logger << "ARP table\n"; 
     arptable.clear();
     
@@ -228,9 +235,7 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
         if(hostname.empty())
             throw FWException("No SNMP hostname specified");
        
-        CHECK_STOP
         c=new SNMPConnection(hostname, community);
-        *logger << "Connecting to " << hostname << "\n";
         try
         {
             c->connect(retries, timeout);
@@ -238,28 +243,26 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
         {
             // If program was stopped, show this, rather
             // than error.
-            CHECK_STOP
             throw;
         }
-        CHECK_STOP
     }
-    *logger << "Connected... " << "\n";
 
     vector<SNMPVariable*> v;
     try
     {
-        *logger << "Pulling atNetAddress table... " << "\n";
+        *logger << "Walking atNetAddress table...\n";
         multimap<string, SNMPVariable* > nw=c->walk(SNMP_AT_TABLE_NET  );
-        *logger << "Pulling atPhysAddress table... " << "\n";
+        CHECK_STOP_AND_THROW_EXCEPTION
+        *logger << "Walking atPhysAddress table...\n";
         multimap<string, SNMPVariable* > pw=c->walk(SNMP_AT_TABLE_PHYS );
+        CHECK_STOP_AND_THROW_EXCEPTION
 
         try
         {
-            CHECK_STOP
 //            int preflen = strlen(SNMP_AT_TABLE_NET);
             for(multimap<string, SNMPVariable* >::iterator j = nw.begin(); j!=nw.end(); ++j)
             {
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
                 if((*j).second->type!=SNMPVariable::snmp_ipaddr)
                 {
                     *logger << "unexpected result type in '" << 
@@ -268,7 +271,7 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
                 }
                 IPAddress ip = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getIPAddressValue();
 
-                *logger << "Learned: " << ip.toString();
+                str << "Learned: " << ip.toString();
 
                 for(multimap<string, SNMPVariable* >::iterator k = pw.begin(); k!=pw.end(); ++k)
                 {
@@ -277,11 +280,13 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
                     {
                         string hwaddr=dynamic_cast<SNMPVariable_String*>(k->second)->toHexString();
                         arptable.insert(make_pair(ip,hwaddr));
-                        *logger << " at " << hwaddr;
+                        str << " at " << hwaddr;
                         break;
                     }
                 }
-                *logger << "\n";
+                *logger << str;
+                
+//                *logger << "\n";
             }
         } catch(...)
         {
@@ -302,7 +307,6 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
             delete c;
         // If program was stopped, show this, rather
         // than error.
-        CHECK_STOP
         throw;
     }
     if(!connection)
@@ -317,6 +321,7 @@ void SNMPQuery::fetchArpTable(Logger *logger,SyncFlag *stop_program, SNMPConnect
  */
 void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPConnection *connection) throw(FWException)
 {
+    std::ostringstream str;
     *logger << "Routing table\n"; 
     routes.clear();
     
@@ -332,9 +337,7 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
         if(hostname.empty())
             throw FWException("No SNMP hostname specified");
         
-        CHECK_STOP
         c=new SNMPConnection(hostname, community);
-        *logger << "Connecting to " << hostname << "\n";
         try
         {
             c->connect(retries, timeout);
@@ -342,64 +345,65 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
         {
             // If program was stopped, show this, rather
             // than error.
-            CHECK_STOP
             throw;
         }
-        CHECK_STOP
     }
+
+    CHECK_STOP_AND_THROW_EXCEPTION
     
     vector<SNMPVariable*> v;
     try
     {
         // Get Indices and destinations
         
-        CHECK_STOP
         multimap<string, SNMPVariable* > w=c->walk(SNMP_ROUTE_DST_TABLE);
+        CHECK_STOP_AND_THROW_EXCEPTION
         try
         {
-            CHECK_STOP
             for(multimap<string, SNMPVariable* >::iterator j = w.begin(); j!=w.end(); ++j)
             {
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
                 if((*j).second->type!=SNMPVariable::snmp_ipaddr)
                 {
-                    *logger << "unexpected result type in '" << 
-                    SNMP_ROUTE_DST_TABLE << "' table. Skipping it.\n";
+                    str << "unexpected result type in '"
+                        << SNMP_ROUTE_DST_TABLE << "' table. Skipping it.\n";
+                    *logger << str;
+                    
                     continue;
                 }
                 IPAddress dst = dynamic_cast<SNMPVariable_IPaddr*>((*j).second)->getIPAddressValue();
                 string rname=(*j).first.substr(strlen(SNMP_ROUTE_DST_TABLE)+1);
 
-                CHECK_STOP
                 v=c->get(string(SNMP_ROUTE_NM_TABLE)+"."+rname);
                 if(v[0]->type!=SNMPVariable::snmp_ipaddr)
                 {
-                    *logger << "unexpected result type in '" << 
-                    SNMP_ROUTE_NM_TABLE << "'table. Skipping it.\n";
+                    str << "unexpected result type in '"
+                        << SNMP_ROUTE_NM_TABLE << "'table. Skipping it.\n";
+                    *logger << str;
+                    
                     SNMPVariable::freeVarList(v);
                     continue;
                 }
                 Netmask nm = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getNetmaskValue();
                 SNMPVariable::freeVarList(v);
 
-                CHECK_STOP
                 v=c->get(string(SNMP_ROUTE_TYPE_TABLE)+"."+rname);
                 long type = SNMPVariable::varList2Int(v);
                 SNMPVariable::freeVarList(v);
 
-                CHECK_STOP
                 v=c->get(string(SNMP_ROUTE_GW_TABLE)+"."+rname);
                 if(v[0]->type!=SNMPVariable::snmp_ipaddr)
                 {
-                    *logger << "unexpected result type in '" << 
-                    SNMP_ROUTE_GW_TABLE << "'table. Skipping it.\n";
+                    str << "unexpected result type in '"
+                        << SNMP_ROUTE_GW_TABLE << "'table. Skipping it.\n";
+                    *logger << str;
+                    
                     SNMPVariable::freeVarList(v);
                     continue;
                 }
                 IPAddress gw = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getIPAddressValue();
                 SNMPVariable::freeVarList(v);
 
-                CHECK_STOP
                 v=c->get(string(SNMP_ROUTE_IF_TABLE)+"."+rname);
                 int ifindex = SNMPVariable::varList2Int(v);
                 SNMPVariable::freeVarList(v);
@@ -409,13 +413,21 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                 Interface *route_intf;
                 if(ici==interfaces.end())
                 {
-                    *logger << "Route references non-existing interface '" << ifindex << "' in '" << 
-                    string(SNMP_ROUTE_IF_TABLE)+"."+rname << "' variable. Ignoring interface reference.\n";
+                    str << "Route references non-existing interface '"
+                        << ifindex << "' in '"
+                        << string(SNMP_ROUTE_IF_TABLE)+"."+rname
+                        << "' variable. Ignoring interface reference.\n";
+                    *logger << str;
+                    
                     route_intf=NULL;
                 } else
                 {
 #ifdef SNMP_CRAWLER_DEBUG
-                    *logger << "Found interface # " << ifindex << " associated with the route: " << string(SNMP_ROUTE_IF_TABLE)+"."+rname << "\n";
+                    str << "Found interface # " << ifindex
+                        << " associated with the route: "
+                        << string(SNMP_ROUTE_IF_TABLE)+"."+rname << "\n";
+                    *logger << str;
+                    
 #endif
                     
                     route_intf=&(*ici).second;
@@ -428,15 +440,17 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
                 IPRoute rt(dst, nm, gw, route_intf, type==SNMP_DIRECT_ROUTE);
                 routes.push_back(rt);
 
-                *logger << "route: " 
-                        << rt.getDestination().toString() 
-                        << "/" << rt.getNetmask().getLength() 
-                        << "  gw " 
-                        << rt.getGateway().toString()
-                        << "  "
-                        << string( (route_intf)?rt.getInterface()->getName():"(interface not found)")
-                        << string( (route_intf && rt.getInterface()->isExt())?"(Ext)":"")
-                        << "\n";
+                str << "route: " 
+                    << rt.getDestination().toString() 
+                    << "/" << rt.getNetmask().getLength() 
+                    << "  gw " 
+                    << rt.getGateway().toString()
+                    << "  "
+                    << string( (route_intf)?rt.getInterface()->getName():"(interface not found)")
+                    << string( (route_intf && rt.getInterface()->isExt())?"(Ext)":"")
+                    << "\n";
+                *logger << str;
+                
             }
         } catch(...)
         {
@@ -455,7 +469,6 @@ void SNMPQuery::fetchRoutingTable(Logger *logger,SyncFlag *stop_program, SNMPCon
             delete c;
         // If program was stopped, show this, rather
         // than error.
-        CHECK_STOP
         throw;
     }
     if(!connection)
@@ -472,6 +485,8 @@ bool SNMPQuery::isDefault(const IPRoute &r) const
 
 void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConnection *connection) throw(FWException)
 {
+    std::ostringstream str;
+
     interfaces.clear();
     
     SNMPConnection *c;
@@ -486,9 +501,7 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
         if(hostname.empty())
             throw FWException("No SNMP hostname specified");
         
-        CHECK_STOP
         c=new SNMPConnection(hostname, community);
-        *logger << "Connecting to " << hostname << "\n";
         try
         {
             c->connect(retries, timeout);
@@ -496,12 +509,12 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
         {
             // If program was stopped, show this, rather
             // than error.
-            CHECK_STOP
             throw;
         }
-        CHECK_STOP
     }
     
+    CHECK_STOP_AND_THROW_EXCEPTION
+            
     multimap<string, SNMPVariable* > w;
     vector<SNMPVariable*>            v;
     try
@@ -510,15 +523,17 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
 
         map<long, list<string> > addr;
         w=c->walk(SNMP_ADDR_INDEX_TABLE);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
 
         for(multimap<string, SNMPVariable* >::iterator j = w.begin(); j!=w.end(); ++j)
         {
-            CHECK_STOP
+            CHECK_STOP_AND_THROW_EXCEPTION
             if((*j).second->type!=SNMPVariable::snmp_int)
             {
-                *logger << "unexpected result type in '" << 
-                SNMP_ADDR_INDEX_TABLE << "' table. Skipping it.\n";
+                str << "unexpected result type in '"
+                    << SNMP_ADDR_INDEX_TABLE << "' table. Skipping it.\n";
+                *logger << str;
+                
                 continue;
             }
             long ind = SNMPVariable::var2Int((*j).second);
@@ -536,49 +551,51 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
  * "ip.ipAddrTable.ipAddrEntry.ipAdEntIfIndex." , ind is an index
  */
             addr[ind].push_back(iname);
-            *logger << "interface #" << ind << ": " << iname << "\n";
+            str << "interface #" << ind << ": " << iname << "\n";
+            *logger << str;
+            
         }
 
         for(multimap<string, SNMPVariable* >::iterator j = w.begin(); j!=w.end(); ++j)
             delete (*j).second;
         w.clear();
 
-        CHECK_STOP
         *logger << "Full interfaces info.\n";
 
         w=c->walk(SNMP_INTERFACE_INDEX);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
         for(multimap<string, SNMPVariable* >::iterator j = w.begin(); j!=w.end(); ++j)
         {
-            CHECK_STOP
+            CHECK_STOP_AND_THROW_EXCEPTION
 
             // Get index
             long ifindex = SNMPVariable::var2Int((*j).second);
-            char str[1024];
+            char oid[1024];
 
             // Get admin status
-            sprintf(str,"%s.%ld", SNMP_INTERFACE_ASTATUS , ifindex);
-            v=c->get(str);
+            sprintf(oid,"%s.%ld", SNMP_INTERFACE_ASTATUS , ifindex);
+            v=c->get(oid);
             if(v.size()!=1)
-                throw FWException(string("Unexpected response length for variable: ")+str);
+                throw FWException(string("Unexpected response length for OID: ")+oid);
             int astatus = SNMPVariable::varList2Int(v);
             SNMPVariable::freeVarList(v);
-            CHECK_STOP
 
             if(!astatus)
             {
-                *logger << "Interface #" << ifindex << " disable by admin - skipping it.\n";
+                str << "Interface #" << ifindex
+                    << " disabled by admin - skipping.\n";
+                *logger << str;
+                
                 continue;
             }
             
             // Get operational status
-            sprintf(str,"%s.%ld", SNMP_INTERFACE_OSTATUS , ifindex);
-            v=c->get(str);
+            sprintf(oid,"%s.%ld", SNMP_INTERFACE_OSTATUS , ifindex);
+            v=c->get(oid);
             if(v.size()!=1)
-                throw FWException(string("Unexpected response length for variable: ")+str);
+                throw FWException(string("Unexpected response length for OID: ")+oid);
             int ostatus = SNMPVariable::varList2Int(v);
             SNMPVariable::freeVarList(v);
-            CHECK_STOP
 
 
 /* gather all information for interface ifindex and create Interface object */
@@ -586,7 +603,9 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
             list<string> &addlist=addr[ifindex];
             if (addlist.empty())
             {
-                *logger << "Interface #" << ifindex << " has no IP address.\n";
+                str << "Interface #" << ifindex << " has no IP address.\n";
+                *logger << str;
+                
 //                continue;
             }
 /*
@@ -601,37 +620,35 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
             }
 */
             // Get desriptions
-            sprintf(str,"%s.%ld", SNMP_INTERFACES_DESCR , ifindex);
-            v=c->get(str);
+            sprintf(oid,"%s.%ld", SNMP_INTERFACES_DESCR , ifindex);
+            v=c->get(oid);
             string descr = SNMPVariable::varList2String(v);
             SNMPVariable::freeVarList(v);
-            CHECK_STOP
 
             // Get physical address
-            sprintf(str,"%s.%ld", SNMP_INTERFACES_PHYSA , ifindex);
-            v=c->get(str);
+            sprintf(oid,"%s.%ld", SNMP_INTERFACES_PHYSA , ifindex);
+            v=c->get(oid);
             if(v.size()!=1)
-                throw FWException(string("Unexpected response length for variable: ")+str);
+                throw FWException(string("Unexpected response length for OID: ")+oid);
             if(v[0]->type!=SNMPVariable::snmp_string)
-                throw FWException(string("Unexpected response type for: ")+str);
+                throw FWException(string("Unexpected response type for: ")+oid);
             string physa = dynamic_cast<SNMPVariable_String*>(v[0])->toHexString();
             SNMPVariable::freeVarList(v);
-            CHECK_STOP
 
             // Get type
-            sprintf(str,"%s.%ld", SNMP_INTERFACES_TYPE , ifindex);
-            v=c->get(str);
+            sprintf(oid,"%s.%ld", SNMP_INTERFACES_TYPE , ifindex);
+            v=c->get(oid);
             if(v.size()!=1)
-                throw FWException(string("Unexpected response length for variable: ")+str);                
+                throw FWException(string("Unexpected response length for OID: ")+oid);                
             int itype = SNMPVariable::varList2Int(v);
             SNMPVariable::freeVarList(v);
-            CHECK_STOP
 
             interfaces[ifindex]=Interface();
             interfaces[ifindex].setName(descr);
             interfaces[ifindex].setOStatus(ostatus);
             interfaces[ifindex].setInterfaceType(itype);
             interfaces[ifindex].setExt(false);
+            snmp_tmp_db.add( &interfaces[ifindex] );
 
             if (physa!="")
                 interfaces[ifindex].setPhysicalAddress(physa);
@@ -641,7 +658,7 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
                 string aprefix= *ali;
 
                 // get netmasks
-                CHECK_STOP
+                CHECK_STOP_AND_THROW_EXCEPTION
                 v=c->get(string(SNMP_NMASK_TABLE)+"."+aprefix);
                 if(v.size()!=1)
                     throw FWException("Can't get netmask");
@@ -649,7 +666,6 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
                     throw FWException("Wrong return type for netmask");
                 string nm = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getNetmaskValue().toString();
                 SNMPVariable::freeVarList(v);
-                CHECK_STOP
             
                 // get addresss
                 v=c->get(string(SNMP_ADDR_TABLE)+"."+aprefix);
@@ -659,22 +675,22 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
                     throw FWException("Wrong return type for IP address");
                 string ad = dynamic_cast<SNMPVariable_IPaddr*>(v[0])->getIPAddressValue().toString();
                 SNMPVariable::freeVarList(v);
-                CHECK_STOP
 
                 v=c->get(string(SNMP_BCAST_TABLE)+"."+aprefix);
                 if(v.size()!=1)
                     throw FWException("Can't get broadcast address");
 //                long bcast_bits = SNMPVariable::varList2Int(v);
                 SNMPVariable::freeVarList(v);
-                CHECK_STOP
 
                 IPv4 *ipaddr=interfaces[ifindex].addIPv4();
                 ipaddr->setAddress(ad);
                 ipaddr->setNetmask(nm);
 
-                *logger << "Adding interface #" << ifindex 
-                        << ": " << ad << "/" << nm
-                        << "\n";
+                str << "Adding interface #" << ifindex 
+                    << ": " << ad << "/" << nm
+                    << "\n";
+                *logger << str;
+                
             }
         } // index walk.
         for(multimap<string, SNMPVariable* >::iterator j = w.begin(); j!=w.end(); ++j)
@@ -694,8 +710,6 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
             delete c;
         // If program was stopped, show this, rather
         // than error.
-        CHECK_STOP
-
         throw;
     }
     if(!connection)
@@ -705,6 +719,7 @@ void SNMPQuery::fetchInterfaces(Logger *logger,SyncFlag *stop_program, SNMPConne
 
 void SNMPQuery::fetchSysInfo(Logger *logger,SyncFlag *stop_program, SNMPConnection *connection) throw(FWException)
 {
+    std::ostringstream str;
     descr     = "" ;
     contact   = "" ;
     location  = "" ;
@@ -718,19 +733,15 @@ void SNMPQuery::fetchSysInfo(Logger *logger,SyncFlag *stop_program, SNMPConnecti
     {
         if(community.empty())
         {
-            CHECK_STOP
             throw FWException("No SNMP community specified");
         }
         
         if(hostname.empty())
         {
-            CHECK_STOP
             throw FWException("No SNMP hostname specified");
         }
         
-        CHECK_STOP
         c=new SNMPConnection(hostname, community);
-        *logger << "Connecting to " << hostname << "\n";
         try
         {
             c->connect(retries, timeout);
@@ -738,10 +749,8 @@ void SNMPQuery::fetchSysInfo(Logger *logger,SyncFlag *stop_program, SNMPConnecti
         {
             // If program was stopped, show this, rather
             // than error.
-            CHECK_STOP
             throw;
         }
-        CHECK_STOP
     }
     
     try
@@ -752,32 +761,31 @@ void SNMPQuery::fetchSysInfo(Logger *logger,SyncFlag *stop_program, SNMPConnecti
         v=c->get(SNMP_SYSNAME);
         sysname    = SNMPVariable::varList2String(v);
         SNMPVariable::freeVarList(v);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
 
         *logger << "Getting Description\n";
         v=c->get(SNMP_SYSDESCR);
         descr    = SNMPVariable::varList2String(v);
         SNMPVariable::freeVarList(v);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
         
         *logger << "Getting Location\n";
         v=c->get(SNMP_LOCATION);
         location = SNMPVariable::varList2String(v);
         SNMPVariable::freeVarList(v);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
         
         *logger << "Getting Contact Info\n";
         v=c->get(SNMP_CONTACT);
         contact  = SNMPVariable::varList2String(v);
         SNMPVariable::freeVarList(v);
-        CHECK_STOP
+        CHECK_STOP_AND_THROW_EXCEPTION
     } catch(...)
     {
         if(!connection)
             delete c;
         // If program was stopped, show this, rather
         // than error.
-        CHECK_STOP
         throw;
     }
     if(!connection)
@@ -1325,11 +1333,18 @@ bool SNMPCrawler::special(const IPNetwork &n) const
 //TODO: multiple threads (via pool).
 void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWException)
 {
+    std::ostringstream str;
     time_t now=time(NULL);
-    *logger << "SNMPCrawler started at " << asctime(localtime(&now)) << ". Seed host: " << (*(queue.begin())).first.toString() << "\n";
+    str << "SNMPCrawler started at " << asctime(localtime(&now))
+        << ". Seed host: " << (*(queue.begin())).first.toString() << "\n";
+    *logger << str;
+    
+    snmp_tmp_db.destroyChildren();
+
     SNMP_discover_query q;
     do {
-        CHECK_STOP
+        CHECK_STOP_AND_RETURN
+
         map<IPAddress,string>::iterator i=queue.begin();
         if(i==queue.end())
         {
@@ -1339,9 +1354,9 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
         string task_phys_address = (*i).second ;
         queue.erase(i);
         
-        CHECK_STOP
-
-        *logger << "Processing " << task.toString() << "\n";
+        str << "\nProcessing " << task.toString() << "\n";
+        *logger << str;
+        
 
         // Now in task we have element to probe
         
@@ -1351,8 +1366,6 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                snmp_timeout
         );
         
-        CHECK_STOP
-
         CrawlerFind *res=new CrawlerFind();
         found[task] = *res; delete res;
         found[task].found_phys_addr = task_phys_address ;
@@ -1363,24 +1376,30 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
         } catch(const FWException &ex)
         {
             // fetch failed
-            *logger << "Failed to fetch ARP table from " << task.toString();
-            if (!ex.toString().empty()) *logger << " : " << ex.toString();
-            *logger << "\n";
+            str << "Failed to fetch ARP table from " << task.toString();
+            if (!ex.toString().empty()) str << " : " << ex.toString();
+            str << "\n";
+            *logger << str;
+            
             continue;
         } catch (std::string s) {
-            *logger << "Failed to fetch ARP table from " << task.toString();
-            *logger << " : " << s;
-            *logger << "\n";
+            str << "Failed to fetch ARP table from " << task.toString();
+            str << " : " << s;
+            str << "\n";
+            *logger << str;
+            
             continue;
         }
-        CHECK_STOP
+
         map<IPAddress, string> at=q.getArtpTable();    
-        *logger << "Got " << long(at.size()) << " entries\n";
+        str << "Got " << long(at.size()) << " entries\n";
+        *logger << str;
+        
 
         int qplus=0, rplus=0, dplus=0;
         for(map<IPAddress, string>::iterator j=at.begin();j!=at.end();++j)
         {
-            CHECK_STOP
+            CHECK_STOP_AND_RETURN
 
             IPAddress  c = (*j).first;
             string    pa = (*j).second;
@@ -1399,9 +1418,24 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
             } else
                 dplus++;
         }
-        if(qplus) *logger << "Adding "   << qplus << " hosts to queue\n"   ;
-        if(rplus) *logger << "Adding "   << rplus << " hosts to results\n" ;
-        if(dplus) *logger << "Skipping " << dplus << " hosts as duplicate, excluded or virtual\n" ;
+        if (qplus)
+        {
+            str << "Adding "   << qplus << " hosts to queue\n"   ;
+            *logger << str;
+            
+        }
+        if (rplus)
+        {
+            str << "Adding "   << rplus << " hosts to results\n" ;
+            *logger << str;
+            
+        }
+        if (dplus)
+        {
+            *logger << "Skipping " << dplus << " hosts as duplicate, excluded or virtual\n" ;
+            *logger << str;
+            
+        }
 
         set<IPAddress> interface_broadcasts;
 
@@ -1422,7 +1456,9 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
 
                 if(!special(net) && included(net.getAddress()) && !point2point(net, NULL))
                 {
-                    *logger << "Network " << string(net) << " found.\n";
+                    str << "Network " << string(net) << " found.\n";
+                    *logger << str;
+                    
                     networks.insert(net);
                 }
             }
@@ -1430,7 +1466,10 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
         } catch(FWException &ex)
         {
             // fetch failed
-            *logger << "Failed to fetch list of interfaces from " << task.toString() << "\n";
+            str << "Failed to fetch list of interfaces from "
+                << task.toString() << "\n";
+            *logger << str;
+            
         }
 
         try
@@ -1445,7 +1484,9 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
         } catch(const FWException &ex)
         {
             // fetch failed
-            *logger << "Failed to fetch sysinfo from " << task.toString() << "\n";
+            str << "Failed to fetch sysinfo from " << task.toString() << "\n";
+            *logger << str;
+            
         }
 
         try
@@ -1519,16 +1560,20 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                             break;
                     }
                     if(have_intf)
-                        *logger << "Guessed that network " << net << 
-                        " is using interface " << intf.getName() << "\n";
+                        str << "Guessed that network " << string(net)
+                            << " is using interface " << intf.getName() << "\n";
+                    *logger << str;
+                    
                 }
 
                 // If route is associated with an interface which is down,
                 // ignore it.
                 if(have_intf && !intf.isUp())
                 {
-                    *logger << "Skipping route for network " << net << 
-                    " which is associated with interface which is currently down.\n";
+                    str << "Skipping route for network " << string(net)
+                        << " which is associated with interface which is currently down.\n";
+                    *logger << str;
+                    
                     continue; 
                 }
 
@@ -1562,9 +1607,12 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                         }
                     } else
                     {
-                        *logger << "Network " << string(net) << " found (via " 
-                                << string(((*j).isDirect())?"direct":"indirect") 
-                                << " route).\n";
+                        str << "Network " << string(net)
+                            << " found (via "
+                            << string(((*j).isDirect())?"direct":"indirect") 
+                            << " route).\n";
+                        *logger << str;
+                        
                         
                         networks.insert(net);
                         nplus++;
@@ -1587,15 +1635,42 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
                 }
             }
 
-            if(qplus) *logger << "Adding "   << qplus << " hosts to queue\n"   ;
-            if(nplus) *logger << "Adding "   << nplus << " networks to results\n"   ;
-            if(rplus) *logger << "Adding "   << rplus << " hosts to results\n" ;
-            if(dplus) *logger << "Skipping " << dplus << " hosts as duplicate, excluded or virtual\n" ;
+            if (qplus)
+            {
+                str << "Adding "   << qplus
+                    << " hosts to queue\n"   ;
+                *logger << str;
+                
+            }
+            if (nplus)
+            {
+                str << "Adding "   << nplus
+                    << " networks to results\n"   ;
+                *logger << str;
+                
+            }
+            if (rplus)
+            {
+                str << "Adding "   << rplus
+                    << " hosts to results\n" ;
+                *logger << str;
+                
+            }
+            if (dplus)
+            {
+                str << "Skipping " << dplus
+                    << " hosts as duplicate, excluded or virtual\n" ;
+                *logger << str;
+                
+            }
 
         } catch(const FWException &ex)
         {
             // fetch failed
-            *logger << "Failed to fetch routing table from " << task.toString() << "\n";
+            str << "Failed to fetch routing table from "
+                << task.toString() << "\n";
+            *logger << str;
+            
         }
 
         // We add interfaces _after_ fetching routing table,
@@ -1611,8 +1686,11 @@ void SNMPCrawler::run_impl(Logger *logger,SyncFlag *stop_program) throw(FWExcept
     if(do_dns)
         bacresolve_results(logger,stop_program);
     
+    snmp_tmp_db.clear(); // at this point all children have been destroyed anyway
+
     now=time(NULL);
-    *logger << "SNMPCrawler - done at " << asctime(localtime(&now)) << "\n";
+    str << "SNMPCrawler - done at " << asctime(localtime(&now)) << "\n";
+    *logger << str;
 }
 
 void SNMPCrawler::remove_virtual(Logger *logger,SyncFlag *stop_program) throw(FWException)
