@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: Compiler.cpp,v 1.25 2006/05/21 04:42:50 vkurland Exp $
+  $Id: Compiler.cpp,v 1.27 2006/09/10 22:40:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -456,7 +456,7 @@ void Compiler::_expandAddressRanges(Rule *rule,FWObject *s)
             for (vector<IPNetwork>::iterator i=vn.begin(); i!=vn.end(); i++)
             {
                 Network *h;
-                h= Network::cast(dbcopy->create(Network::TYPENAME,true) );
+                h= Network::cast(dbcopy->create(Network::TYPENAME) );
                 h->setName(string("%n-")+a1.toString()+string("%") );
                 h->setNetmask(i->getNetmask());
                 h->setAddress(i->getAddress());
@@ -772,7 +772,7 @@ bool Compiler::Begin::processNext()
         {
             Rule *rule=Rule::cast(*i);
 
-            Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName(),true) );
+            Rule  *r= Rule::cast(compiler->dbcopy->create(rule->getTypeName()) );
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
 
@@ -780,6 +780,7 @@ bool Compiler::Begin::processNext()
         }
         init=true;
         if (!name.empty()) cout << " " << name << endl << flush;
+
         return true;
     }
     return false;
@@ -896,7 +897,7 @@ bool Compiler::splitIfRuleElementMatchesFW::processNext()
 	    cl.push_back(o);   // can not remove right now because remove invalidates iterator
             nre--;
 
-	    Rule  *new_rule= Rule::cast(compiler->dbcopy->create(rule->getTypeName(),true) );
+	    Rule  *new_rule= Rule::cast(compiler->dbcopy->create(rule->getTypeName()) );
 	    compiler->temp_ruleset->add(new_rule);
 	    new_rule->duplicate(rule);
             RuleElement *new_re=RuleElement::cast(new_rule->getFirstByType(re_type));
