@@ -6,7 +6,7 @@
 
   Author:  Vadim Kurland     vadim@vk.crocodile.org
 
-  $Id: PolicyCompiler.cpp,v 1.13 2006/06/15 05:05:02 vkurland Exp $
+  $Id: PolicyCompiler.cpp,v 1.16 2006/09/10 22:40:37 vkurland Exp $
 
   This program is free software which we release under the GNU General Public
   License. You may redistribute and/or modify this program under the terms
@@ -180,6 +180,9 @@ bool PolicyCompiler::checkForShadowing(const PolicyRule &r1,const PolicyRule &r2
     Address  *dst2=getFirstDst(&r2);
     Service  *srv2=getFirstSrv(&r2);
 
+    if (MultiAddressRunTime::isA(src1) || MultiAddressRunTime::isA(dst1) ||
+        MultiAddressRunTime::isA(src2) || MultiAddressRunTime::isA(dst2))
+        return false;
 
     if (src1==NULL || dst1==NULL || srv1==NULL)
         throw FWException("Can not compare rules because rule "+r1.getLabel()+" has a group in one of its elements. Aborting.");
@@ -259,7 +262,13 @@ bool PolicyCompiler::ItfNegation::processNext()
 
     list<FWObject*> allInterfaces=compiler->fw->getByType(Interface::TYPENAME);
 
-    RuleElementItf *itfre=rule->getItf();   assert(itfre);
+    RuleElementItf *itfre=rule->getItf();
+    if (itfre==NULL)
+        compiler->abort("Missing interface RE in rule '" +
+                        rule->getLabel() +
+                        "' id=" + rule->getId()
+        );
+
     if (itfre->getNeg())
     {
         for (FWObject::iterator i=itfre->begin(); i!=itfre->end(); ++i)
@@ -305,7 +314,7 @@ bool  PolicyCompiler::InterfacePolicyRules::processNext()
             if (FWReference::cast(o)!=NULL)
                 o=FWReference::cast(o)->getPointer();
             PolicyRule *r= PolicyRule::cast(
-                compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                compiler->dbcopy->create(PolicyRule::TYPENAME) );
             r->duplicate(rule);
             r->setInterfaceId(o->getId());
             compiler->temp_ruleset->add(r);
@@ -513,7 +522,7 @@ bool PolicyCompiler::splitServices::processNext()
         list<Service*> &sl=(*i1).second;
 
         PolicyRule *r= PolicyRule::cast(
-            compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            compiler->dbcopy->create(PolicyRule::TYPENAME) );
         compiler->temp_ruleset->add(r);
         r->duplicate(rule);
         RuleElementSrv *nsrv=r->getSrv();
@@ -560,7 +569,7 @@ bool PolicyCompiler::separateTCPWithFlags::processNext()
         if ( s->inspectFlags() )
         {
             PolicyRule *r= PolicyRule::cast(
-        	compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+        	compiler->dbcopy->create(PolicyRule::TYPENAME) );
             compiler->temp_ruleset->add(r);
             r->duplicate(rule);
             RuleElementSrv *nsrv=r->getSrv();
@@ -731,7 +740,7 @@ bool PolicyCompiler::ConvertToAtomicForAddresses::processNext()
 	for (FWObject::iterator i2=dst->begin(); i2!=dst->end(); ++i2) {
 
 	    PolicyRule *r = PolicyRule::cast(
-		compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+		compiler->dbcopy->create(PolicyRule::TYPENAME) );
 	    r->duplicate(rule);
 	    compiler->temp_ruleset->add(r);
 
@@ -764,7 +773,7 @@ bool PolicyCompiler::ConvertToAtomicForIntervals::processNext()
     for (FWObject::iterator i1=ivl->begin(); i1!=ivl->end(); ++i1) {
 
         PolicyRule *r = PolicyRule::cast(
-            compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+            compiler->dbcopy->create(PolicyRule::TYPENAME) );
         r->duplicate(rule);
         compiler->temp_ruleset->add(r);
         
@@ -794,7 +803,7 @@ bool  PolicyCompiler::ConvertToAtomic::processNext()
 	    for (FWObject::iterator i3=srv->begin(); i3!=srv->end(); i3++)
             {
                 PolicyRule *r = PolicyRule::cast(
-                    compiler->dbcopy->create(PolicyRule::TYPENAME,false) );
+                    compiler->dbcopy->create(PolicyRule::TYPENAME) );
                 r->duplicate(rule);
                 compiler->temp_ruleset->add(r);
 
